package dk.vajhoej.vms.call.lib;

import dk.vajhoej.record.Alignment;
import dk.vajhoej.record.FieldType;
import dk.vajhoej.record.RecordException;
import dk.vajhoej.record.Struct;
import dk.vajhoej.record.StructField;
import dk.vajhoej.record.StructInfoCache;
import dk.vajhoej.vms.call.RFA;

import static dk.vajhoej.vms.call.lib.VMSCodes.*;

/**
 * Class RAB represents a RAB (Record Access Block) block.
 */
@Struct(alignment=Alignment.PACKED)
public class RAB {
	@StructField(n=0,type=FieldType.INT1)
	private int bid;
	@StructField(n=1,type=FieldType.INT1)
	private int bln;
	@StructField(n=2,type=FieldType.INT2)
	private int isi;
	@StructField(n=3,type=FieldType.INT4)
	private int rop;
	@StructField(n=4,type=FieldType.INT4)
	private int sts;
	@StructField(n=5,type=FieldType.INT4)
	private int stv;
	@StructField(n=6,type=FieldType.STRUCT)
	private RFA rfa;
	@StructField(n=7,type=FieldType.INT2)
	private int fill1;
	@StructField(n=8,type=FieldType.INT4)
	private int ctx;
	@StructField(n=9,type=FieldType.INT2)
	private int fill2;
	@StructField(n=10,type=FieldType.INT1)
	private int rac;
	@StructField(n=11,type=FieldType.INT1)
	private int tmo;
	@StructField(n=12,type=FieldType.INT2)
	private int usz;
	@StructField(n=13,type=FieldType.INT2)
	private int rsz;
	@StructField(n=14,type=FieldType.INT4)
	private int ubf;
	@StructField(n=15,type=FieldType.INT4)
	private int rbf;
	@StructField(n=16,type=FieldType.INT4)
	private int rhb;
	@StructField(n=17,type=FieldType.INT4)
	private int kbfpbf;
	@StructField(n=18,type=FieldType.INT1)
	private int kszpsz;
	@StructField(n=19,type=FieldType.INT1)
	private int krf;
	@StructField(n=20,type=FieldType.INT1)
	private int mbf;
	@StructField(n=21,type=FieldType.INT1)
	private int mbc;
	@StructField(n=22,type=FieldType.INT4)
	private int bktdct;
	@StructField(n=23,type=FieldType.INT4)
	private int fab;
	@StructField(n=24,type=FieldType.INT4)
	private int xab;
	private transient FAB fabblk;
	private transient byte[] usrbuf;
	private transient byte[] recbuf;
	private static int RABLEN;
	static {
		try {
			RABLEN = StructInfoCache.analyze(RAB.class).getLength();
		} catch (RecordException e) {
			throw new RuntimeException("Internal problem: " + e);
		}
	}
	public RAB() {
		this.bid = RAB$C_BID;
		this.bln = RABLEN;
		this.isi = 0;
		this.ctx = 0;
		this.ubf = 0;
		this.rbf = 0;
		this.rhb = 0;
		this.kbfpbf = 0;
		this.rfa = new RFA();
		this.fab = 0;
		this.xab = 0;
		this.fabblk = null;
		this.usrbuf = null;
		this.recbuf = null;
	}
	public RAB(FAB fabblk) {
		this();
		this.fabblk = fabblk;
	}
	public RAB(FAB fabblk, int rop, int rac) {
		this(fabblk);
		this.rop = rop;
		this.rac = rac;
	}
	public int getRop() {
		return rop;
	}
	public void setRop(int rop) {
		this.rop = rop;
	}
	public RFA getRfa() {
		return rfa;
	}
	public void setRfa(RFA rfa) {
		this.rfa = rfa;
	}
	public int getRac() {
		return rac;
	}
	public void setRac(int rac) {
		this.rac = rac;
	}
	public int getTmo() {
		return tmo;
	}
	public void setTmo(int tmo) {
		this.tmo = tmo;
	}
	public int getUsz() {
		return usz;
	}
	public void setUsz(int usz) {
		this.usz = usz;
	}
	public int getRsz() {
		return rsz;
	}
	public void setRsz(int rsz) {
		this.rsz = rsz;
	}
	public int getUbf() {
		return ubf;
	}
	public void setUbf(int ubf) {
		this.ubf = ubf;
	}
	public int getRbf() {
		return rbf;
	}
	public void setRbf(int rbf) {
		this.rbf = rbf;
	}
	public FAB getFabblk() {
		return fabblk;
	}
	public void setFabblk(FAB fabblk) {
		this.fabblk = fabblk;
	}
	public byte[] getUsrbuf() {
		return usrbuf;
	}
	public void setUsrbuf(byte[] usrbuf) {
		this.usz = usrbuf.length;
		this.usrbuf = usrbuf;
	}
	public byte[] getRecbuf() {
		return recbuf;
	}
	public void setRecbuf(byte[] recbuf) {
		this.rsz = recbuf.length;
		this.recbuf = recbuf;
	}
}
