/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.test;

import static org.junit.Assert.*;

import org.junit.Test;

import dk.vajhoej.record.FieldType;
import dk.vajhoej.record.Struct;
import dk.vajhoej.record.StructField;
import dk.vajhoej.vms.call.VMS.Block;
import dk.vajhoej.vms.call.VMS.BlockHeader;
import dk.vajhoej.vms.call.VMS.MultiBlock;

public class TestBlock {
	@Struct
	public static class Block1 {
		@StructField(n=0,type=FieldType.INT4)
		private int a;
		@StructField(n=1,type=FieldType.INT4)
		private int b;
		public Block1() {
			this(0, 0);
		}
		public Block1(int a, int b) {
			this.a = a;
			this.b = b;
		}
		public int getA() {
			return a;
		}
		public int getB() {
			return b;
		}
	}
	@Struct
	public static class Block2 {
		@StructField(n=0,type=FieldType.INT4)
		private int c;
		public Block2() {
			this(0);
		}
		public Block2(int a) {
			this.c = a;
		}
		public int getC() {
			return c;
		}
	}
	@Test
	public void testBlock() {
        try {
        	byte[] expected = { 1, 0, 0, 0, 2, 0, 0, 0 };
        	Block<Block1> blk = new Block<Block1>(new Block1(1, 2));
        	byte[] b = blk.getValue();
        	assertEquals("Length", expected.length, b.length);
        	for(int i = 0; i <b.length; i++) {
        		assertEquals("Byte #" + i, expected[i], b[i]);
        	}
        	blk.update();
        	Block1 o = blk.getObject(Block1.class);
        	assertEquals("A", 1, o.getA());
        	assertEquals("B", 2, o.getB());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
	@Test
	public void testMultiBlock() {
        try {
        	byte[] expected = { 2, 0, 0, 0, 4, 0, 0, 0, 8, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 0, 0, 0 };
        	MultiBlock blk = new MultiBlock(new BlockHeader(4, 8), new Block1(1, 2), new Block2(3));
        	byte[] b = blk.getValue();
        	assertEquals("Length", expected.length, b.length);
        	for(int i = 0; i <b.length; i++) {
        		assertEquals("Byte #" + i, expected[i], b[i]);
        	}
        	blk.update();
        	Block1 o1 = blk.getObject(0, Block1.class);
        	assertEquals("A", 1, o1.getA());
        	assertEquals("B", 2, o1.getB());
        	Block2 o2 = blk.getObject(1, Block2.class);
        	assertEquals("C", 3, o2.getC());
        } catch(Exception ex) {
        	fail("Unexpected exception: " + ex);
        }
	}
}
